﻿using Infragistics.Web.UI.GridControls;
using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;

namespace WHDGDynamicColumns
{
    public partial class WebForm1 : System.Web.UI.Page
    {
        protected void Page_Load(object sender, EventArgs e)
        {
            if (Session["data"] == null)
                Session["data"] = CreateDataSource();

            AddColumnsToGrid();
            AddBands();

            AddCustomSummary(true);
            this.WebHierarchicalDataGrid1.CalculateCustomSummary += WebDataGrid1_CalculateCustomSummary;

            WebHierarchicalDataGrid1.DataSource = CreateDataSource();
            WebHierarchicalDataGrid1.DataBind();
        }

        private void AddColumnsToGrid()
        {
            WebHierarchicalDataGrid1.Columns.Clear();
            AddColumn("id", "System.Int32");
            AddColumn("Item", "System.Int32");
            WebHierarchicalDataGrid1.DataKeyFields = "id";
            WebHierarchicalDataGrid1.DataMember = "Module";
        }

        private void AddBands()
        {
            Band childBand = new Band();
            childBand.AutoGenerateColumns = false;
            childBand.Key = "TRN_CODE";
            childBand.DataMember = "TRN_CODE";
            childBand.DataKeyFields = "ChildId";
            WebHierarchicalDataGrid1.Bands.Add(childBand);
            childBand.Columns.Clear();
            AddColumn("ChildId", "System.Int32", childBand);
            AddColumn("ItemCh", "System.Int32", childBand);
            AddColumn("ParentId", "System.Int32", childBand);
            AddColumn("ChView", "System.Boolean", childBand);
        }

        private void AddColumn(string columnKey, string dataType, Band band = null)
        {
            BoundDataField column = new BoundDataField();

            column.Key = columnKey;
            column.DataFieldName = columnKey;
            column.DataType = dataType;
            column.Header.Text = columnKey + " Header";
            if(band == null)
            {
                WebHierarchicalDataGrid1.Columns.Add(column);     
            } else
            {
                band.Columns.Add(column);
            }
        }

        protected object WebDataGrid1_CalculateCustomSummary(object sender, CustomSummaryEventArgs e)
        {
            if(e.Summary.CustomSummaryName == "Custom")
            {
                return "Custom Result";
            }
            return null;
        }


        protected void AddCustomSummary(bool isAdd)
        {

            SummaryRowSetting itemColumnSetting = new SummaryRowSetting(this.WebHierarchicalDataGrid1.GridView, "Item");
            SummarySetting customSetting = new SummarySetting();
            customSetting.SummaryType = SummaryType.Custom;
            customSetting.CustomSummaryName = "Custom";
            customSetting.FormatString = "Custom = {1}";
            itemColumnSetting.SummarySettings.Add(customSetting);
            WebHierarchicalDataGrid1.Behaviors.SummaryRow.ColumnSettings.Add(itemColumnSetting);

            ColumnSummaryInfo customInfo = new ColumnSummaryInfo();
            customInfo.ColumnKey = "Item";
            Summary stdevSummary = new Summary();
            stdevSummary.SummaryType = SummaryType.Custom;
            stdevSummary.CustomSummaryName = "Custom";
            customInfo.Summaries.Add(stdevSummary);
            WebHierarchicalDataGrid1.Behaviors.SummaryRow.ColumnSummaries.Add(customInfo);

            this.WebHierarchicalDataGrid1.RefreshBehaviors();

        }

        private DataSet CreateDataSource()
        {
            DataSet ds = new DataSet();
            ds.Tables.Add(CreateGridDataSourceForModules("Module"));
            ds.Tables.Add(CreateGridDataSourceForSubModules("TRN_CODE", true));
            DataRelation dr = new DataRelation("id", ds.Tables["Module"].Columns["id"], ds.Tables["TRN_CODE"].Columns["ParentId"], false);
            ds.Relations.Add(dr);
            return ds;
        }

        private DataTable CreateGridDataSourceForModules(string tableName)
        {
            System.Data.DataTable table = new DataTable(tableName);
            DataColumn column;
            DataRow row;
            column = new DataColumn();
            column.DataType = System.Type.GetType("System.Int32");
            column.ColumnName = "id";
            column.ReadOnly = true;
            table.Columns.Add(column);
            column = new DataColumn();
            column.DataType = System.Type.GetType("System.Int32");
            column.ColumnName = "Item";
            column.AutoIncrement = false;
            column.Caption = "Item";
            column.ReadOnly = false;
            column.Unique = false;
            table.Columns.Add(column);
            DataColumn[] PrimaryKeyColumns = new DataColumn[1];
            PrimaryKeyColumns[0] = table.Columns["id"];
            table.PrimaryKey = PrimaryKeyColumns;

            for (int i = 0; i <= 10 - 1; i++)
            {
                row = table.NewRow();
                row["id"] = i;
                row["Item"] = i;
                table.Rows.Add(row);
            }

            return table;
        }

        private DataTable CreateGridDataSourceForSubModules(string tableName, bool duplicate)
        {
            System.Data.DataTable table = new DataTable(tableName);
            DataColumn column;
            DataRow row;
            column = new DataColumn();
            column.DataType = System.Type.GetType("System.Int32");
            column.ColumnName = "ChildId";
            column.AutoIncrement = false;
            column.Caption = "ChildId";
            column.ReadOnly = false;
            column.Unique = false;
            table.Columns.Add(column);
            column = new DataColumn();
            column.DataType = System.Type.GetType("System.Int32");
            column.ColumnName = "ItemCh";
            column.AutoIncrement = false;
            column.Caption = "ItemCh";
            column.ReadOnly = false;
            column.Unique = false;
            table.Columns.Add(column);
            column = new DataColumn();
            column.DataType = System.Type.GetType("System.Int32");
            column.ColumnName = "ParentId";
            column.ReadOnly = true;
            table.Columns.Add(column);
            column = new DataColumn();
            column.DataType = System.Type.GetType("System.Boolean");
            column.ColumnName = "ChView";
            column.AutoIncrement = false;
            column.Caption = "View";
            column.Unique = false;
            column.DefaultValue = false;
            table.Columns.Add(column);
            column = new DataColumn();
            column.DataType = System.Type.GetType("System.Boolean");
            column.ColumnName = "ChUpdate";
            column.AutoIncrement = false;
            column.ReadOnly = false;
            column.Unique = false;
            column.DefaultValue = false;
            table.Columns.Add(column);
            column = new DataColumn();
            column.DataType = System.Type.GetType("System.Boolean");
            column.ColumnName = "ChDelete";
            column.AutoIncrement = false;
            column.Caption = "Delete";
            column.Unique = false;
            column.DefaultValue = false;
            table.Columns.Add(column);
            column = new DataColumn();
            column.DataType = System.Type.GetType("System.Boolean");
            column.ColumnName = "ChSave";
            column.AutoIncrement = false;
            column.Caption = "Save";
            column.ReadOnly = false;
            column.Unique = false;
            column.DefaultValue = false;
            table.Columns.Add(column);

            if (!duplicate)
            {
                DataColumn[] PrimaryKeyColumns = new DataColumn[1];
                PrimaryKeyColumns[0] = table.Columns["id"];
                table.PrimaryKey = PrimaryKeyColumns;
            }
            else
            {
                DataColumn[] PrimaryKeyColumns = new DataColumn[1];
                PrimaryKeyColumns[0] = table.Columns["ChildId"];
                table.PrimaryKey = PrimaryKeyColumns;
            }

            for (int i = 0; i <= 5 - 1; i++)
            {
                row = table.NewRow();
                row["ParentId"] = i;
                row["ItemCh"] = i*10;
                row["ChildId"] = i + 10;
                table.Rows.Add(row);
            }

            return table;
        }
    }
}